var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import * as React from 'react';
import debounce from '../debounce';
import { componentDefaultProps } from './component-common';
import ErrorMessages from './error-messages';
import Help from './help';
import InputControl from './controls/input';
import InputGroup from './input-group';
import Row from './row';
var defaultProps = __assign(__assign({}, componentDefaultProps), { 
    // eslint-disable-next-line @typescript-eslint/no-unused-vars, @typescript-eslint/no-empty-function
    blurCallback: function (name, value) { }, blurDebounceInterval: 0, changeDebounceInterval: 500, className: '', elementRef: React.createRef(), 
    // eslint-disable-next-line @typescript-eslint/no-unused-vars, @typescript-eslint/no-empty-function
    keyDownCallback: function (event) { }, required: false, type: 'text', updateOnBlur: true, updateOnChange: true, value: '' });
var Input = /** @class */ (function (_super) {
    __extends(Input, _super);
    function Input(props) {
        var _this = _super.call(this, props) || this;
        _this.setValue = function (value) {
            if (_this.state.incomingPropValue !== value) {
                _this.blurDebounced.cancel();
                _this.changeDebounced.cancel();
                _this.props.onSetValue(value);
            }
        };
        _this.handleBlur = function (event) {
            var value = event.currentTarget.value;
            var _a = _this.props, updateOnBlur = _a.updateOnBlur, isPristine = _a.isPristine, propValue = _a.value, blurCallback = _a.blurCallback, name = _a.name;
            _this.setState({ currentValue: value });
            if (updateOnBlur) {
                _this.changeDebounced.cancel();
                if (isPristine) {
                    // should update as we have just left a pristine input
                    _this.blurDebounced(value);
                }
                else if (propValue !== value) {
                    // should update because the value has changed
                    _this.blurDebounced(value);
                }
            }
            blurCallback(name, value);
        };
        _this.handleChange = function (event) {
            var value = event.currentTarget.value;
            var _a = _this.props, updateOnChange = _a.updateOnChange, changeCallback = _a.changeCallback, name = _a.name;
            _this.setState({ currentValue: value });
            if (updateOnChange) {
                _this.changeDebounced(value);
            }
            changeCallback(name, value);
        };
        _this.handleKeyDown = function (event) {
            var keyDownCallback = _this.props.keyDownCallback;
            if (event.key === 'Enter') {
                _this.changeDebounced.flush();
            }
            keyDownCallback(event);
        };
        var value = props.value, changeDebounceInterval = props.changeDebounceInterval, blurDebounceInterval = props.blurDebounceInterval;
        _this.state = {
            currentValue: value,
            incomingPropValue: value,
            valueIsChanging: false,
        };
        _this.changeDebounced = debounce(_this.setValue, changeDebounceInterval);
        _this.blurDebounced = debounce(_this.setValue, blurDebounceInterval);
        return _this;
    }
    Input.getDerivedStateFromProps = function (props, state) {
        var incomingPropValue = props.value;
        if (incomingPropValue !== state.incomingPropValue) {
            return {
                valueIsChanging: true,
                incomingPropValue: incomingPropValue,
            };
        }
        return null;
    };
    Input.prototype.shouldComponentUpdate = function (nextProps, nextState) {
        var _this = this;
        var valueIsChanging = nextState.valueIsChanging, incomingPropValue = nextState.incomingPropValue;
        if (valueIsChanging === true) {
            this.setState({
                valueIsChanging: false,
                currentValue: incomingPropValue,
            }, function () {
                _this.setValue(incomingPropValue);
            });
            return false;
        }
        return true;
    };
    Input.prototype.render = function () {
        var _a = this.props, 
        /* eslint-disable @typescript-eslint/no-unused-vars */
        blurCallback = _a.blurCallback, blurDebounceInterval = _a.blurDebounceInterval, changeCallback = _a.changeCallback, changeDebounceInterval = _a.changeDebounceInterval, isPristine = _a.isPristine, keyDownCallback = _a.keyDownCallback, name = _a.name, onSetValue = _a.onSetValue, updateOnBlur = _a.updateOnBlur, updateOnChange = _a.updateOnChange, value = _a.value, 
        /* eslint-enable @typescript-eslint/no-unused-vars */
        addonAfter = _a.addonAfter, addonBefore = _a.addonBefore, buttonAfter = _a.buttonAfter, buttonBefore = _a.buttonBefore, className = _a.className, elementRef = _a.elementRef, elementWrapperClassName = _a.elementWrapperClassName, errorMessages = _a.errorMessages, help = _a.help, id = _a.id, label = _a.label, labelClassName = _a.labelClassName, layout = _a.layout, required = _a.required, rowClassName = _a.rowClassName, showErrors = _a.showErrors, type = _a.type, passthoughProps = __rest(_a, ["blurCallback", "blurDebounceInterval", "changeCallback", "changeDebounceInterval", "isPristine", "keyDownCallback", "name", "onSetValue", "updateOnBlur", "updateOnChange", "value", "addonAfter", "addonBefore", "buttonAfter", "buttonBefore", "className", "elementRef", "elementWrapperClassName", "errorMessages", "help", "id", "label", "labelClassName", "layout", "required", "rowClassName", "showErrors", "type"]);
        var markAsInvalid = showErrors && (errorMessages.length > 0 || required);
        var control = (React.createElement(InputControl, __assign({}, passthoughProps, { className: markAsInvalid ? "is-invalid " + className : className, id: id, onBlur: this.handleBlur, onChange: this.handleChange, onKeyDown: this.handleKeyDown, elementRef: elementRef, type: type, value: this.state.currentValue })));
        if (type === 'hidden') {
            return control;
        }
        var inputGroupProps = {
            addonAfter: addonAfter,
            addonBefore: addonBefore,
            buttonAfter: buttonAfter,
            buttonBefore: buttonBefore,
        };
        var rowProps = {
            elementWrapperClassName: elementWrapperClassName,
            labelClassName: labelClassName,
            rowClassName: rowClassName,
            label: label,
            errorMessages: errorMessages,
            htmlFor: id,
            layout: layout,
            required: required,
            showErrors: showErrors,
        };
        if (addonBefore || addonAfter || buttonBefore || buttonAfter) {
            control = React.createElement(InputGroup, __assign({}, inputGroupProps), control);
        }
        if (layout === 'elementOnly') {
            return control;
        }
        return (React.createElement(Row, __assign({}, rowProps),
            React.createElement(React.Fragment, null,
                control,
                help ? React.createElement(Help, { help: help }) : null,
                showErrors ? React.createElement(ErrorMessages, { messages: errorMessages }) : null)));
    };
    Input.defaultProps = defaultProps;
    return Input;
}(React.Component));
export default Input; // as React.ComponentClass<InputProps>;
//# sourceMappingURL=input.js.map